@@ -1,4 +1,4 @@
-## Copyright (C) 2018  Brodie Gaslam
+## Copyright (C) 2020  Brodie Gaslam
 ##
 ## This file is part of "fansi - ANSI Control Sequence Aware String Functions"
 ##
@@ -23,18 +23,16 @@
 #' stops you will need to provide them yourself if you are using anything
 #' outside of the standard tab stop every 8 characters that is the default.
 #'
-#' @note Non-ASCII strings are converted to and returned in UTF-8 encoding.
+#' @note Non-ASCII strings are converted to and returned in UTF-8 encoding.  The
+#'   `ctl` parameter only affects which _Control Sequences_ are considered zero
+#'   width.  Tabs will always be converted to spaces, irrespective of the `ctl`
+#'   setting.
 #' @seealso [fansi] for details on how _Control Sequences_ are
 #'   interpreted, particularly if you are getting unexpected results.
 #' @export
 #' @inheritParams substr_ctl
 #' @param x character vector or object coercible to character; any tabs therein
 #'   will be replaced.
-#' @param tab.stops integer(1:n) indicating position of tab stops to use
-#'   when converting tabs to spaces.  If there are more tabs in a line than
-#'   defined tab stops the last tab stop is re-used.  For the purposes of
-#'   applying tab stops, each input line is considered a line and the character
-#'   count begins from the beginning of the input line.
 #' @return character, `x` with tabs replaced by spaces, with elements
 #'   possibly converted to UTF-8.
 #' @examples
@@ -57,7 +55,8 @@
 #' ) )
 
 tabs_as_spaces <- function(
-  x, tab.stops=getOption('fansi.tab.stops'), warn=getOption('fansi.warn')
+  x, tab.stops=getOption('fansi.tab.stops'), warn=getOption('fansi.warn'),
+  ctl='all'
 ) {
   if(!is.character(x)) x <- as.character(x)
   if(!is.logical(warn)) warn <- as.logical(warn)
@@ -66,16 +65,29 @@ tabs_as_spaces <- function(
   if(!is.numeric(tab.stops) || !length(tab.stops) || any(tab.stops < 1))
     stop("Argument `tab.stops` must be numeric and strictly positive")
 
+  if(!is.character(ctl))
+    stop("Argument `ctl` must be character.")
+  ctl.int <- integer()
+  if(length(ctl)) {
+    # duplicate values in `ctl` are okay, so save a call to `unique` here
+    if(anyNA(ctl.int <- match(ctl, VALID.CTL)))
+      stop(
+        "Argument `ctl` may contain only values in `",
+        deparse(VALID.CTL), "`"
+      )
+  }
   term.cap.int <- seq_along(VALID.TERM.CAP)
   .Call(
-    FANSI_tabs_as_spaces, enc2utf8(x), as.integer(tab.stops), warn, term.cap.int
+    FANSI_tabs_as_spaces, enc2utf8(x), as.integer(tab.stops), warn,
+    term.cap.int, ctl.int
   )
 }
 #' Test Terminal Capabilities
 #'
 #' Outputs ANSI CSI SGR formatted text to screen so that you may visually
-#' inspect what color capabilities your terminal supports.  The three tested
-#' terminal capabilities are:
+#' inspect what color capabilities your terminal supports.
+#'
+#' The three tested terminal capabilities are:
 #'
 #' * "bright" for bright colors with SGR codes in 90-97 and 100-107
 #' * "256" for colors defined by "38;5;x" and "48;5;x" where x is in 0-255
@@ -93,6 +105,12 @@ tabs_as_spaces <- function(
 #' modes, and also tests for truecolor support via the $COLORTERM system
 #' variable.
 #'
+#' Functions with the `term.cap` parameter like `substr_ctl` will warn if they
+#' encounter 256 or true color SGR sequences and `term.cap` indicates they are
+#' unsupported as such a terminal may misinterpret those sequences.  Bright
+#' codes in terminals that do not support them are more likely to be silently
+#' ignored, so `fansi` functions do not warn about those.
+#'
 #' @seealso [fansi] for details on how _Control Sequences_ are
 #'   interpreted, particularly if you are getting unexpected results.
 #' @export
@@ -215,7 +233,8 @@ html_code_block <- function(x, class='fansi-output') {
 #' finally [`html_code_block`].
 #'
 #' If you require more control than this function provides you can set the
-#' `knitr` hooks manually with `knitr::knit_hooks$set`.
+#' `knitr` hooks manually with `knitr::knit_hooks$set`.  If you are seeing your
+#' output gaining extra line breaks, look at the `split.nl` option.
 #'
 #' @note Since we do not formally import the `knitr` functions we do not
 #'   guarantee that this function will always work properly with `knitr` /
@@ -225,7 +244,8 @@ html_code_block <- function(x, class='fansi-output') {
 #' @seealso [`has_sgr`], [`sgr_to_html`], [`html_esc`], [`html_code_block`],
 #'   [`knitr` output hooks](https://yihui.name/knitr/hooks/#output-hooks),
 #'   [embedding CSS in
-#'   Rmd](https://bookdown.org/yihui/rmarkdown/language-engines.html#javascript-and-css).
+#'   Rmd](https://bookdown.org/yihui/rmarkdown/language-engines.html#javascript-and-css),
+#'   and the vignette `vignette(package='fansi', 'sgr-in-rmd')`.
 #' @param hooks list, this should the be `knitr::knit_hooks` object; we
 #'   require you pass this to avoid a run-time dependency on `knitr`.
 #' @param which character vector with the names of the hooks that should be
@@ -242,6 +262,16 @@ html_code_block <- function(x, class='fansi-output') {
 #'   HTML <STYLE> tags as a side effect.  The default value is designed to
 #'   ensure that there is no visible gap in background color with lines with
 #'   height 1.5 (as is the default setting in `rmarkdown` documents v1.1).
+#' @param split.nl TRUE or FALSE (default), set to TRUE to split input strings
+#'   by any newlines they may contain to avoid any newlines inside SPAN tags
+#'   created by [sgr_to_html()].  Some markdown->html renders can be configured
+#'   to convert embedded newlines into line breaks, which may lead to a doubling
+#'   of line breaks.  With the default `proc.fun` the split strings are
+#'   recombined by [html_code_block()], but if you provide your own `proc.fun`
+#'   you'll need to account for the possibility that the character vector it
+#'   receives will have a different number of elements than the chunk output.
+#'   This argument only has an effect if chunk output contains ANSI CSI SGR
+#'   sequences.
 #' @param .test TRUE or FALSE, for internal testing use only.
 #' @return named list with the prior output hooks for each of `which`.
 #' @examples
@@ -287,6 +317,7 @@ set_knit_hooks <- function(
     html_code_block(sgr_to_html(html_esc(x)), class=class),
   class=sprintf("fansi fansi-%s", which),
   style=getOption("fansi.css"),
+  split.nl=FALSE,
   .test=FALSE
 ) {
   if(
@@ -325,6 +356,8 @@ set_knit_hooks <- function(
 
   if(!is.character(style))
     stop("Argument `style` must be character.")
+  if(!isTRUE(split.nl %in% c(TRUE, FALSE)))
+    stop("Argument `split.n` must be TRUE or FALSE")
 
   old.hook.list <- vector('list', length(which))
   names(old.hook.list) <- which
@@ -334,14 +367,16 @@ set_knit_hooks <- function(
   base.err <-
     "are you sure you passed `knitr::knit_hooks` as the `hooks` argument?"
 
-  make_hook <- function(old.hook, class) {
+  make_hook <- function(old.hook, class, split.nl) {
     force(old.hook)
     force(class)
+    force(split.nl)
     function(x, options) {
       # If the output has SGR in it, then convert to HTML and wrap
       # in PRE/CODE tags
 
       if(any(has_sgr(x))) {
+        if(split.nl) x <- unlist(strsplit_sgr(x, '\n', fixed=TRUE))
         res <- try(proc.fun(x=x, class=class))
         if(inherits(res, "try-error"))
           stop(
@@ -375,7 +410,7 @@ set_knit_hooks <- function(
       )
       break
     }
-    new.hook.list[[i]] <- make_hook(old.hook, class[[i]])
+    new.hook.list[[i]] <- make_hook(old.hook, class[[i]], split.nl)
     old.hook.list[[i]] <- old.hook
   }
   if(
