@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018  Brodie Gaslam
+ * Copyright (C) 2020  Brodie Gaslam
  *
  * This file is part of "fansi - ANSI Control Sequence Aware String Functions"
  *
@@ -23,17 +23,17 @@ static const
 R_CallMethodDef callMethods[] = {
   {"has_csi", (DL_FUNC) &FANSI_has, 3},
   {"strip_csi", (DL_FUNC) &FANSI_strip, 3},
-  {"strwrap_csi", (DL_FUNC) &FANSI_strwrap_ext, 14},
-  {"state_at_pos_ext", (DL_FUNC) &FANSI_state_at_pos_ext, 7},
+  {"strwrap_csi", (DL_FUNC) &FANSI_strwrap_ext, 15},
+  {"state_at_pos_ext", (DL_FUNC) &FANSI_state_at_pos_ext, 8},
   {"process", (DL_FUNC) &FANSI_process_ext, 1},
   {"check_assumptions", (DL_FUNC) &FANSI_check_assumptions, 0},
   {"digits_in_int", (DL_FUNC) &FANSI_digits_in_int_ext, 1},
-  {"tabs_as_spaces", (DL_FUNC) &FANSI_tabs_as_spaces_ext, 4},
+  {"tabs_as_spaces", (DL_FUNC) &FANSI_tabs_as_spaces_ext, 5},
   {"color_to_html", (DL_FUNC) &FANSI_color_to_html_ext, 1},
   {"esc_to_html", (DL_FUNC) &FANSI_esc_to_html, 3},
-  {"unhandled_esc", (DL_FUNC) &FANSI_unhandled_esc, 1},
+  {"unhandled_esc", (DL_FUNC) &FANSI_unhandled_esc, 2},
   {"unique_chr", (DL_FUNC) &FANSI_unique_chr, 1},
-  {"nzchar_esc", (DL_FUNC) &FANSI_nzchar, 4},
+  {"nzchar_esc", (DL_FUNC) &FANSI_nzchar, 5},
   {"add_int", (DL_FUNC) &FANSI_add_int_ext, 2},
   {"strsplit", (DL_FUNC) &FANSI_strsplit, 3},
   {"cleave", (DL_FUNC) &FANSI_cleave, 1},
@@ -43,10 +43,12 @@ R_CallMethodDef callMethods[] = {
   {"set_int_max", (DL_FUNC) &FANSI_set_int_max, 1},
   {"get_int_max", (DL_FUNC) &FANSI_get_int_max, 0},
   {"check_enc", (DL_FUNC) &FANSI_check_enc_ext, 2},
-  {"what_as_int", (DL_FUNC) &FANSI_what_as_int_ext, 1},
+  {"ctl_as_int", (DL_FUNC) &FANSI_ctl_as_int_ext, 1},
   {NULL, NULL, 0}
 };
 
+SEXP FANSI_warn_sym;
+
 void R_init_fansi(DllInfo *info)
 {
  /* Register the .C and .Call routines.
